from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from unittest2 import TestCase
import os.path


def mail_asset(name, ext='txt'):
    tests_dir_path = os.path.dirname(__file__)
    filename = '.'.join((name, ext))
    return open(os.path.join(tests_dir_path, 'mails', filename))


class TestMailView(TestCase):
    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    @browsing
    def test_heading_is_subject(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'Die B\xfcrgschaft', plone.first_heading())

    @browsing
    def test_From_header(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'from@example.org',
                          browser.css('.mailFrom td').first.text)

    @browsing
    def test_Subject_header(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'Die B\xfcrgschaft',
                          browser.css('.mailSubject td').first.text)

    @browsing
    def test_Date_header(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'Jan 01, 1970 01:00 AM',
                          browser.css('.mailDate td').first.text)

    @browsing
    def test_To_header(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'Friedrich H\xf6lderlin <to@example.org>',
                          browser.css('.mailTo td').first.text)

    @browsing
    def test_CC_header(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('latin1')))
        browser.login().visit(mail)
        self.assertEquals(u'cc@example.org',
                          browser.css('.mailCc td').first.text)

    @browsing
    def test_body(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('utf8')))
        browser.login().visit(mail)
        body_lines = browser.css('.mailBody').first.text.split('\n')
        self.assertIn(u'Friedrich Schiller', body_lines)
        self.assertIn(u'\xbbDie Stadt vom Tyrannen befreien!\xab', body_lines)

    @browsing
    def test_attachments(self, browser):
        mail = create(Builder('mail')
                      .with_message(mail_asset('nested_attachments')))
        browser.login().visit(mail)

        # Plone <= 4.1 uses "kB",
        # Plone >= 4.2 uses "KB"
        # Therefore we just lowercase it in the test because it is not relevant.

        self.assertItemsEqual(
            ['1703693_0412c29a4f.jpg file 134.4 kb',
             '3512536451_e1310bf568.jpg file 218.3 kb'],
            map(str.lower, browser.css('.mailAttachment').text))

    @browsing
    def test_mail_body_is_html_safe(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('xxs_mail')))
        browser.login().visit(mail)

        self.assertNotIn("alert('hello')", browser.contents,
                         "Javascript gets not removed by the mail view.")

    @browsing
    def test_mail_body_contains_all_html_parts(self, browser):
        """Regression:
        """
        mail = create(Builder('mail').with_message(mail_asset('multiple_html_parts')))
        browser.login().visit(mail)
        self.assertEquals('Hello World',
                          browser.css('.mailBody').first.text)

    @browsing
    def test_style_blocks_get_parsed(self, browser):
        mail = create(Builder('mail').with_message(mail_asset('xxs_mail')))
        browser.login().visit(mail)

        self.assertEquals(
            'color:red; size:18px',
            browser.css('.mailBody h1').first.get('style'))

        self.assertNotIn(
            '<style>', browser.contents,
            '<style> tags gets not wrapped correctly with the wrapper class.')




# mail messages may contain alternative representations of a message.
# the following list defines the order which alternative is chosen.
PREFERRED_MULTIPART_ALTERNATIVES = [
    'multipart/mixed', 
    'multipart/digest',
    'multipart/related',
    'text/html',
    'text/plain',
]

EXIT_CODES = {
    'NOINPUT': 66,   # cannot open input
    'DATAERR': 65,   # data format error
    'NOUSER':  67,   # addressee unknown
    'CANTCREAT': 73, # can't create (user) output file
    'NOPERM': 77,    # permission denied 
}


# various utility methods for handling e-mail messages
from BeautifulSoup import BeautifulSoup
from DocumentTemplate.DT_Util import html_quote
from email.header import decode_header
from email.Utils import mktime_tz, parsedate_tz
from ftw.mail import config
import re


# a regular expression that matches src attributes of img tags containing a cid
IMG_SRC_RE = re.compile(r'<img[^>]*?src="cid:([^"]*)', re.IGNORECASE|re.DOTALL)
BODY_RE = re.compile(r'<body>(.*)</body>', re.IGNORECASE|re.DOTALL)
APPLE_PARTIAL_ENCODING_RE = re.compile(r'^"(.*=\?.*\?=.*)"( <.*>)$')
ENCODED_WORD_WITHOUT_LWSP = re.compile(r'(=\?.*?\?=)(\r\n)([ \t])')


def safe_decode_header(value):
    """ Handles rfc 2047 encoded header with non-ascii characters.
    Always returns an utf-8 encoded string.
    """

    if value is None:
        return None

    if isinstance(value, unicode):
        value = value.encode('utf-8')

    new_value = []

    # When sending mails with Apple Mail, which have umlauts in
    # From-/To-/Cc-Headers, it encodes the name of the person but it
    # wraps it into quotes.
    # The python email module does not support this wrapping, so we
    # remove the quotes in this situation.
    # Example:
    # From: "=?iso-8859-1?Q?Boss_H=FCgo?=" <hugo@boss.com>
    apple_partial_encoding = APPLE_PARTIAL_ENCODING_RE.match(value)
    if apple_partial_encoding:
        value = ''.join(apple_partial_encoding.groups())

    # Fix up RFC 2047 encoded words separated by 'CRLF LWSP' (which is fine
    # according to the RFC) by replacing the CRLF with a SPACE so
    # decode_header can parse them correctly.
    # This works around a bug in decode_header that has been fixed in 3.3.
    # See http://bugs.python.org/issue4491 and its duplicate.
    # Example:
    # From: =?utf-8?Q?B=C3=A4rengraben?=\r\n <from@example.org>
    value = re.sub(ENCODED_WORD_WITHOUT_LWSP, '\\1 \\3', value)

    for data, charset in decode_header(value):
        if charset is not None and charset not in ('utf-8', 'utf8'):
            data = data.decode(charset).encode('utf-8')
        elif charset is None:
            data = safe_utf8(data)
        new_value.append(data and data or '')

    return ' '.join(new_value).rstrip()


def get_header(msg, name):
    """Returns the value of the named header field of a mail message.
    Handles rfc 2047 encoded header with non-ascii characters.
    Always returns an utf-8 encoded string.
    """
    value = ''
    if msg is not None and msg.has_key(name):
        value = safe_decode_header(msg.get(name))
    return value


def get_date_header(msg, name):
    """ Returns an UTC timestamp from a header field containing a date.
    Compensates for the timezone difference if the header contains
    timezone information.
    """
    value = get_header(msg, name)
    ts = 0.0
    try:
        ts = mktime_tz(parsedate_tz(value))
    except TypeError:
        pass
    return ts


def get_payload(msg):
    """Get the decoded message payload as utf-8 string"""

    # get encoding for the msg object, use utf-8 as fallback encoding
    # for msg objects without a correct charset information
    encoding = msg.get_content_charset('utf-8')
    payload = msg.get_payload(decode=1)
    try:
        payload = payload.decode(encoding)
    except UnicodeDecodeError:
        payload = payload.decode('iso-8859-1', 'replace')
    payload = payload.encode('utf-8')
    return payload


def get_body(msg, url_prefix=''):
    """Returns the mail body as HTML string. All text parts of a multipart
    message are returned."""
    result = []
    for part in get_text_payloads(msg):
        result.append(adjust_image_tags(part, msg, url_prefix))
    return result


def get_attachments(msg):
    """ Returns a list describing the attachements. Only attachments with
    a filename are returned."""
    attachments = []
    if msg is not None and msg.is_multipart():
        for position,part in enumerate(msg.walk()):
            content_type = part.get_content_type()
            filename = get_filename(part)
            if filename is None:
                continue
            # determine size
            size = 0
            if content_type == 'message/rfc822':
                size = len(part.as_string())
            else:
                size = len(part.get_payload(decode=1))
            attachments.append({'filename': filename,
                                'content-type': content_type,
                                'size': size,
                                'position': position})
    return attachments


def remove_attachments(msg, positions):
    """Remove all attachments which have position listed in `positions`
    from the email Message `msg`.
    Returns the same email Message object without attachments.
    If `clone` is True, the message will be copied first
    """

    if msg is None:
        raise ValueError('Cannot delete attachments from email message None')

    if 0 in positions:
        raise ValueError('Cannot delete the message itself (position 0)')

    if len(positions) == 0:
        # no attachments should be removed - so just skip
        return msg

    if not msg.is_multipart():
        raise ValueError('Email message is not multipart - there are '
                         'no attachments.')

    # get the parts to delete filtering the position
    parts_to_delete = [part for pos, part in enumerate(msg.walk())
                           if pos in positions]

    if len(positions) != len(parts_to_delete):
        raise ValueError('One or more attachments could not be found.')

    def _recursive_remove_parts(msg):
        """Recursive function for removing payloads (attachments).
        """
        if not msg.is_multipart():
            return msg

        new_payload = []
        for part in msg.get_payload():
            if part not in parts_to_delete:
                new_payload.append(_recursive_remove_parts(part))
        msg.set_payload(new_payload)
        return msg

    # do it
    return _recursive_remove_parts(msg)


def get_text_payloads(msg):
    """Go recursivly through the message parts and return a list of all
    text parts in HTML format"""
    if msg is None:
        return []
    parts = []
    if msg.is_multipart():
        msgs = msg.get_payload()
        # we only want the best alternative
        if msg.get_content_type() == 'multipart/alternative':
            alternatives = []
            for part in msgs:
                alternatives.append(part.get_content_type())
            best_pos = get_best_alternative(alternatives)
            parts += get_text_payloads(msgs[best_pos])
        # we go through all parts
        else:
            for part in msgs:
                parts += get_text_payloads(part)
    # not a multipart message
    else:
        payload = get_payload(msg)
        if payload:
            if msg.get_content_type() == 'text/html':
                parts.append(payload)
            elif msg.get_content_type() == 'text/plain':
                parts.append(text2html(payload))
    return parts


def adjust_image_tags(html, msg, url_prefix):
    """Adjust image tags of the given HTML string which reference an image by
    Content-Id. The src attribute is set to the attachment of the given
    message.

    """
    matches = IMG_SRC_RE.findall(html)
    for m in matches:
        pos = get_position_for_cid(msg, m)
        if pos:
            download_url = '%s/get_attachment?position=%s' % (url_prefix, pos)
            html = html.replace('cid:' + m, download_url)
    return html


def get_position_for_cid(msg, cid):
    """Return the position of the message part with the given Content-Id."""
    position = -1
    cid = '<' + cid + '>'
    for part in msg.walk():
        position += 1
        if part.get('Content-Id', None) == cid:
            return position
    return None


def get_filename(msg):
    """Get the filename of a message (part)
    """
    filename = msg.get_filename(None)

    if filename is None:
        filename = msg.get_param('Name', None)

    # if the value is already decoded or another tuple
    # we just take the value and use the decode_header function
    if isinstance(filename, tuple):
        if filename[0] is not None and filename[0] not in ('utf-8', 'utf8'):
            filename = filename[2].decode(filename[0]).encode('utf-8')
        else:
            filename = safe_utf8(filename[2])

    else:
        filename = safe_decode_header(filename)

    return filename


def get_best_alternative(alternatives):
    """Get the index of the most preferred alternative in the given list of
    alternatives.
    """
    for content_type in config.PREFERRED_MULTIPART_ALTERNATIVES:
        if content_type in alternatives:
            return alternatives.index(content_type)
    return 0


def text2html(text):
    """Replaces all line breaks with a br tag, and wraps it in a p tag.
    """
    return '<p>%s</p>' % html_quote(text.strip()).replace('\n', '<br />')


def unwrap_html_body(html, css_class=None):
    """ Return the content of the body tag for inline display in another
    html document.
    """
    soup = BeautifulSoup(html, fromEncoding='utf8')
    if soup.body:
        soup = soup.body
    body_soup = BeautifulSoup('<div>%s</div>' % soup.renderContents(), fromEncoding='utf8')
    if css_class:
        body_soup.div['class'] = css_class
    body_style = soup.get('style')
    if body_style:
        body_soup.div['style'] = body_style
    return body_soup.renderContents()


def unwrap_attached_msg(msg):
    """ If a msg contains an attachmed message return the attached one.
    """
    if msg.is_multipart():
        for part in msg.walk():
            content_type = part.get_content_type()
            if content_type == 'message/rfc822':
                return part.get_payload(0)
    return msg


def safe_utf8(text):
    """Returns an utf-8 encoded version of the given string with unknown
    encoding.
    """
    encodings = ('utf8', 'iso-8859-1', 'iso-8859-15')
    for enc in encodings:
        try:
            text = text.decode(enc)
            return text.encode('utf8')
        except UnicodeDecodeError:
            pass
    return text.decode('ascii', 'replace').encode('utf8')



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getFSVersionTuple
from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from unittest2 import TestCase
from unittest2 import skipUnless
import os


def asset(filename):
    here = os.path.dirname(__file__)
    path = os.path.join(here, 'mails', filename)
    with open(path, 'r') as file_:
        return file_.read()


def catalog_indexdata_of(obj):
    catalog = getToolByName(obj, 'portal_catalog')
    path = '/'.join(obj.getPhysicalPath())
    return catalog.getIndexDataForUID(path)


class TestSearchableText(TestCase):

    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def test_plain_text_body(self):
        mail = create(Builder('mail')
                      .with_message(asset('latin1.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('tyrannen', searchable_text)

    def test_utf8_encoded_body(self):
        mail = create(Builder('mail')
                      .with_message(asset('utf8.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('tyrannen', searchable_text)

    @skipUnless(getFSVersionTuple() < (4, 3), "Plone < 4.3")
    def test_umlauts_Plone_4_2_and_older(self):
        # The text contains the umlauts "äöu", which is indexed
        # as umlauts in Plone < 4.3
        mail = create(Builder('mail')
                      .with_message(asset('attachment.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('äöü', searchable_text)

        # Now test with a latin1 encoded message part
        mail = create(Builder('mail')
                      .with_message(asset('latin1_multipart.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('b\xc3\xbcrost\xc3\xbchle', searchable_text)

    @skipUnless(getFSVersionTuple() >= (4, 3), "Plone >= 4.3")
    def test_umlauts_Plone_4_3_and_newer(self):
        # The text contains the umlauts "äöu", which is indexed
        # without diacritics (as "aou")  in Plone >= 4.3
        mail = create(Builder('mail')
                      .with_message(asset('attachment.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('aou', searchable_text)

        # Now test with a latin1 encoded message part
        mail = create(Builder('mail')
                      .with_message(asset('latin1_multipart.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('burostuhle', searchable_text)

    def test_attached_email(self):
        mail = create(Builder('mail')
                      .with_message(asset('fwd_attachment.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('consectetuer', searchable_text)

    def test_nested_attached_html_file(self):
        mail = create(Builder('mail')
                      .with_message(asset('nested_html_attachment.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertIn('consectetuer', searchable_text)

    def test_html_tags_are_not_indexed___converted(self):
        mail = create(Builder('mail')
                      .with_message(asset('latin1.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertNotIn('div', searchable_text)

    def test_html_tags_are_not_indexed(self):
        mail = create(Builder('mail')
                      .with_message(asset('fwd_attachment.txt')))
        searchable_text = catalog_indexdata_of(mail)['SearchableText']
        self.assertNotIn('div', searchable_text)



from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from ftw.workspace.interfaces import IWorkspaceLayer
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility
from zope.interface import alsoProvides
import json
import os.path


def mail_asset(name, ext='txt'):
    tests_dir_path = os.path.dirname(__file__)
    filename = '.'.join((name, ext))
    return open(os.path.join(tests_dir_path, 'mails', filename))


class TestMailTab(TestCase):

    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestMailTab, self).setUp()

        self.portal = self.layer.get('portal')

    def test_mail_tab_available(self):
        alsoProvides(self.portal.REQUEST, IWorkspaceLayer)
        view = self.portal.restrictedTraverse('tabbedview_view-mails')

        self.assertTrue(view, 'Mail tab is not available')

    def test_mail_date_parsing(self):
        create(Builder('mail').with_message(mail_asset('latin1')))
        mail_row = self.get_mails_tab_data().get('rows')[0]
        self.assertEquals('01.01.1970 01:00', mail_row.get('Date'))

    def test_mail_invalid_date_results_in_empty_string(self):
        create(Builder('mail').with_message(mail_asset('invalid_date')))
        mail_row = self.get_mails_tab_data().get('rows')[0]
        self.assertEquals('', mail_row.get('Date'))

    def test_mail_date_parsing_with_time_zone(self):
        create(Builder('mail').with_message(mail_asset('time_zone_dates')))
        mail_row = self.get_mails_tab_data().get('rows')[0]
        self.assertEquals('28.08.2010 18:50', mail_row.get('Date'))

    def get_mails_tab_data(self):
        request = self.portal.REQUEST
        alsoProvides(request, IWorkspaceLayer)
        registry = getUtility(IRegistry)
        registry['ftw.tabbedview.interfaces.ITabbedView.extjs_enabled'] = True
        request.form['tableType'] = 'extjs'
        view = self.portal.restrictedTraverse('tabbedview_view-mails')
        return json.loads(view())



#!/usr/bin/python

import os
import sys
import urllib


def post_message(url, recipient, message_txt):
    """ post an email message to the given url
    """

    if not url:
        print "Invalid url."
        print "usage: mta2plone.py url <recipient>"
        sys.exit(64)

    data = {'mail': message_txt}
    if recipient and len(recipient) > 0:
        data['recipient'] = recipient

    try:
        result = urllib.urlopen(url, urllib.urlencode(data)).read()
    except (IOError, EOFError), e:
        print "ftw.mail error: could not connect to server", e
        sys.exit(73)

    try:
        exitcode, errormsg = result.split(':')
        if exitcode != '0':
            print 'Error %s: %s' % (exitcode, errormsg)
            sys.exit(int(exitcode))
    except ValueError:
        print 'Unknown error.'
        sys.exit(69)

    sys.exit(0)


if __name__ == '__main__':
    # This gets called by the MTA when a new message arrives.
    # The mail message file gets passed in on the stdin

    # Get the raw mail
    message_txt = sys.stdin.read()

    url = ''
    if len(sys.argv) > 1:
        url = sys.argv[1]

    recipient = ''
    # If mta2plone is executed as external command by the MTA, the
    # environment variable ORIGINAL_RECIPIENT contains the entire
    # recipient address, before any address rewriting or aliasing
    recipient = os.environ.get('ORIGINAL_RECIPIENT')

    if len(sys.argv) > 2:
        recipient = sys.argv[2]

    post_message(url, recipient, message_txt)



from ftw.upgrade import UpgradeStep
from Products.Archetypes import config


class AddReferenceablebehavior(UpgradeStep):
    """Add referenceablebehavior.
    """

    def __call__(self):
        self.install_upgrade_profile()
        msg = 'Update UID catalog after enabling referenceablebehavior on mails.'
        uidcatalog = self.getToolByName(config.UID_CATALOG)
        for mail in self.objects({'portal_type': 'ftw.mail.mail'}, msg):
            uidcatalog.catalog_object(mail, '/'.join(mail.getPhysicalPath()))



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.interfaces import IEmailAddress
from ftw.mail.interfaces import IMailSettings
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.publisher.browser import TestRequest
import os


def asset(filename):
    here = os.path.dirname(__file__)
    path = os.path.join(here, 'mails', filename)
    with open(path, 'r') as file_:
        return file_.read()


class TestInboundMail(TestCase):

    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestInboundMail, self).setUp()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager', ])

        wftool = getToolByName(self.portal, 'portal_workflow')
        wftool.setChainForPortalTypes(('Folder',),
                                      ('simple_publication_workflow',))

        self.folder = create(Builder('folder'))

        here = os.path.dirname(__file__)
        self.ascii = open(os.path.join(here, 'mails', 'ascii_7bit.txt'), 'r').read()
        self.resent = open(os.path.join(here, 'mails', 'resent.txt'), 'r').read()
        # setup recipient and sender email addresses for test user
        mtool = getToolByName(self.portal, 'portal_membership')
        user = mtool.getAuthenticatedMember()
        user.setMemberProperties(dict(email='from@example.org'))

        emailaddress = IEmailAddress(TestRequest())
        self.mail_to = emailaddress.get_email_for_object(self.folder)

        logout()

    def test_no_message(self):
        view = self.portal.restrictedTraverse('@@mail-inbound')
        self.assertEquals('66:No mail message supplied.', view())

    def test_invalid_message(self):
        request = TestRequest(mail=1)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('65:Invalid mail message supplied.', view())

    def test_sender(self):
        request = TestRequest(mail=self.ascii)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('from@example.org', view.sender())
        request = TestRequest(mail=self.resent)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('resent.from@example.org', view.sender())

    def test_recipient(self):
        request = TestRequest(mail=self.ascii)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('to@example.org', view.recipient())
        request = TestRequest(mail=self.resent)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('resent.to@example.org', view.recipient())

    def test_recipient_from_request(self):
        request = TestRequest(mail=self.ascii)
        request.form['recipient'] = 'recipient@example.org'
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('recipient@example.org', view.recipient())
        request = TestRequest(mail=self.resent)
        request.form['recipient'] = 'recipient@example.org'
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('recipient@example.org', view.recipient())

    def test_unknown_sender(self):
        # unknown sender
        msg_txt = 'To: to@example.org\n'\
            'From: unknown@example.org\n'\
            'Subject: Test'
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('77:Unknown sender. Permission denied.', view())
        # known upper-case sender, lower-case member email
        msg_txt = 'To: %s\n'\
            'From: FROM@example.org\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

        # known lower-case sender, upper-case member email
        login(self.portal, TEST_USER_NAME)
        mtool = getToolByName(self.portal, 'portal_membership')
        user = mtool.getAuthenticatedMember()
        user.setMemberProperties(dict(email='FROM@example.org'))
        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

    def test_unknown_destination(self):
        msg_txt = 'To: unknown@example.org\n'\
            'From: from@example.org\n'\
            'Subject: Test'
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('73:Destination does not exist.', view())

    def test_mail_creation(self):
        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

    def test_long_header(self):
        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: A long mail header with more than 78 characters.'\
            'Lorem ipsum dolor sit amet, consectetur adipisicing elit,'\
            ' sed do eiusmod tempor incididunt ut labore et dolore '\
            'magna aliqua.' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

    def test_weird_characters_in_subject(self):
        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: Here comes a tab  and some umlauts \xc3\xa4\xc3\xb6\xc3\xbc' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

    def test_inbound_view_returns_text_plain(self):
        msg_txt = 'To: %s\n'\
            'From: FROM@example.org\n'\
            'Subject: Test' % self.mail_to
        self.portal.REQUEST.set("mail", msg_txt)
        view = self.portal.restrictedTraverse("mail-inbound")
        view()
        self.assertEquals(
            "text/plain",
            self.portal.REQUEST.response.getHeader('Content-Type'))

    def test_no_sender_email(self):
        msg_txt = 'To: %s\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('77:Could not extract sender email.', view())

    def test_nested_mail_is_unwrapped(self):
        create(Builder('user').with_email('fwd.from@example.org')
               .with_roles('Contributor', on=self.folder))
        mail = asset('fwd_attachment.txt')
        mail = mail.replace('To: fwd.to@example.org', 'To: %s' % self.mail_to)

        request = TestRequest(mail=mail)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

        obj = self.folder.get('lorem-ipsum')
        self.assertEquals('Lorem Ipsum', obj.Title())

    def test_nested_mail_no_unwrapping(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IMailSettings)
        settings.unwrap_mail = False

        create(Builder('user').with_email('fwd.from@example.org')
               .with_roles('Contributor', on=self.folder))
        mail = asset('fwd_attachment.txt')
        mail = mail.replace('To: fwd.to@example.org', 'To: %s' % self.mail_to)

        request = TestRequest(mail=mail)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')
        self.assertEquals('0:OK', view())

        obj = self.folder.get('fwd-lorem-ipsum')
        self.assertEquals('Fwd: Lorem Ipsum', obj.Title())

    def test_unauthorized_to_add_mails(self):
        self.folder.manage_permission('ftw.mail: Add Inbound Mail',
                                      roles=[], acquire=False)

        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')

        self.assertEquals('77:Unable to create message. Permission denied.',
                          view())

    def test_mails_not_addable(self):
        ttool = getToolByName(self.portal, 'portal_types')
        ttool.get('ftw.mail.mail').global_allow = False

        msg_txt = 'To: %s\n'\
            'From: from@example.org\n'\
            'Subject: Test' % self.mail_to
        request = TestRequest(mail=msg_txt)
        view = getMultiAdapter((self.portal, request), name='mail-inbound')

        self.assertEquals('77:Disallowed subobject type. Permission denied.',
                          view())



from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.emailaddress import UUIDEmailAddress
from ftw.mail.interfaces import IEmailAddress
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.uuid.interfaces import IUUID
from unittest2 import TestCase
from zope.interface.verify import verifyClass
from zope.publisher.browser import TestRequest


class TestUUIDResolver(TestCase):

    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer.get('portal')

        setRoles(self.portal, TEST_USER_ID, ['Manager', 'Member'])

        self.subfolder = create(Builder('folder'))

        self.request = TestRequest()
        self.emailaddress = IEmailAddress(self.request)

    def test_verify_adapter(self):
        verifyClass(IEmailAddress, UUIDEmailAddress)

    def test_get_object_for_email(self):
        email = "%s@example.org" % IUUID(self.subfolder)

        self.assertEquals(self.subfolder,
                          self.emailaddress.get_object_for_email(email))

    def test_get_email_for_object(self):
        email = "%s@example.org" % IUUID(self.subfolder)
        self.assertEquals(
            email,
            self.emailaddress.get_email_for_object(self.subfolder))

    def test_get_email_for_object_with_different_domain(self):
        email = "%s@otherdomain.org" % IUUID(self.subfolder)
        self.assertEquals(
            email,
            self.emailaddress.get_email_for_object(self.subfolder,
                                                   'otherdomain.org'))



from AccessControl import getSecurityManager
from AccessControl import Unauthorized
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from Acquisition import aq_base
from Acquisition import aq_inner
from email.Utils import parseaddr
from ftw.mail import exceptions
from ftw.mail import utils
from ftw.mail.interfaces import IEmailAddress
from ftw.mail.interfaces import IMailInbound
from ftw.mail.interfaces import IMailSettings
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import createContent
from plone.dexterity.utils import iterSchemata
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize import instance
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from z3c.form.interfaces import IValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.container.interfaces import INameChooser
from zope.interface import implements
from zope.schema import getFields
from zope.schema import getFieldsInOrder
from zope.security.interfaces import IPermission
import email


class MailInbound(BrowserView):
    """ A view that handles inbound mail posted by mta2plone.py
    """

    implements(IMailInbound)

    def __call__(self):
        return self.render()

    def render(self):
        self.request.response.setHeader('Content-Type', 'text/plain')
        try:
            self.inbound()
            return '0:OK'
        except exceptions.MailInboundException, e:
            return str(e)

    def inbound(self):
        msg = self.msg()
        settings = getUtility(IRegistry).forInterface(IMailSettings)
        if settings.unwrap_mail:
            # if we find an attached mail, use this instead of the whole one
            msg = utils.unwrap_attached_msg(msg)

        user = self.get_user()
        sm = getSecurityManager()
        newSecurityManager(self.request, user)
        try:
            destination = self.get_destination()
            createMailInContainer(destination, msg.as_string())
        except Unauthorized:
            raise exceptions.PermissionDenied(self.msg(), user)
        except ValueError:
            raise exceptions.DisallowedSubobjectType(self.msg(), user)
        finally:
            setSecurityManager(sm)

    def get_user(self):
        sender_email = self.sender()
        if not sender_email:
            raise exceptions.NoSenderFound(self.msg())

        acl_users = getToolByName(self.context, 'acl_users')
        pas_search = getMultiAdapter((self.context, self.request),
                                     name='pas_search')
        users = pas_search.searchUsers(email=sender_email)
        if len(users) > 0:
            user = acl_users.getUserById(users[0].get('userid'))
            if not hasattr(user, 'aq_base'):
                user = user.__of__(acl_users)
            return user
        else:
            raise exceptions.UnknownSender(self.msg())

    def get_destination(self):
        emailaddress = IEmailAddress(self.request)
        destination = emailaddress.get_object_for_email(self.recipient())

        if destination is None:
            raise exceptions.DestinationDoesNotExist(self.recipient())
        return destination

    @instance.memoize
    def msg(self):
        """ create an email.Message instance from request.
        """
        msg_txt = self.request.get('mail', None)
        if msg_txt is None:
            raise exceptions.NoInput()
        try:
            msg = email.message_from_string(msg_txt)
        except TypeError:
            raise exceptions.InvalidMessage()
        return msg

    @instance.memoize
    def sender(self):
        """ get the email address of the sender
        """
        sender = utils.get_header(self.msg(), 'Resent-From')
        if not sender:
            sender = utils.get_header(self.msg(), 'From')
        (sender_name, sender_address) = parseaddr(sender)
        return sender_address

    @instance.memoize
    def recipient(self):
        """ get the email address of the recipient
        """
        # If recipient is supplied in request (by mta2plone), use that
        recipient = self.request.get('recipient')
        if not recipient:
            # Otherwise fall back to `Resent-To` or `To` headers
            recipient = utils.get_header(self.msg(), 'Resent-To')
            if not recipient:
                recipient = utils.get_header(self.msg(), 'To')
        (recipient_name, recipient_address) = parseaddr(recipient)
        return recipient_address


def createMailInContainer(container, message):
    """Add a mail object to a container.

    The new object, wrapped in its new acquisition context, is returned.
    """

    # lookup the type of the 'message' field and create an instance
    fti = getUtility(IDexterityFTI, name='ftw.mail.mail')
    schema = fti.lookupSchema()
    field_type = getFields(schema)['message']._type
    message_value = field_type(data=message,
                       contentType='message/rfc822', filename=u'message.eml')
    # create mail object
    content = createContent('ftw.mail.mail', message=message_value)

    container = aq_inner(container)
    container_fti = container.getTypeInfo()

    # check permission
    permission = queryUtility(IPermission, name='ftw.mail.AddInboundMail')
    if permission is None:
        raise Unauthorized("Cannot create %s" % content.portal_type)
    if not getSecurityManager().checkPermission(permission.title, container):
        raise Unauthorized("Cannot create %s" % content.portal_type)

    # check addable types
    if container_fti is not None and \
            not container_fti.allowType(content.portal_type):
        raise ValueError("Disallowed subobject type: %s" % (
                content.portal_type))

    normalizer = queryUtility(IIDNormalizer)
    normalized_subject = normalizer.normalize(content.title)

    name = INameChooser(container).chooseName(normalized_subject, content)
    content.id = name

    newName = container._setObject(name, content)
    obj = container._getOb(newName)
    obj = set_defaults(obj, container)
    obj.reindexObject()
    return obj


def set_defaults(obj, container):
    """set the default value for all fields on the mail object
    (including additional behaviors)"""

    for schema in iterSchemata(obj):
        for name, field in getFieldsInOrder(schema):
            # Remove acquisition wrapper when getting field value so
            # determining if a field is already set works as expected
            value = field.get(field.interface(aq_base(obj)))
            if value == field.missing_value or value is None:
                # No value is set, so we try to set the default value
                # otherwise we set the missing value
                default = queryMultiAdapter((
                        container,
                        container.REQUEST,  # request
                        None,  # form
                        field,
                        None,  # Widget
                        ), IValue, name='default')
                if default is not None:
                    default = default.get()
                if default is None:
                    default = getattr(field, 'default', None)
                if default is None:
                    try:
                        default = field.missing_value
                    except:
                        pass
                field.set(field.interface(obj), default)
    return obj



from ftw.builder import Builder
from ftw.builder import create
from ftw.mail.mail import IMail
from ftw.mail.testing import FTW_MAIL_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from plone.namedfile import NamedBlobFile
from plone.rfc822.interfaces import IPrimaryFieldInfo
from plone.uuid.interfaces import IUUID
from unittest2 import TestCase
from zExceptions import NotFound
from zope.component import createObject
from zope.component import queryUtility
import os


class TestMailIntegration(TestCase):

    layer = FTW_MAIL_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestMailIntegration, self).setUp()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager', 'Member'])

        here = os.path.dirname(__file__)
        self.msg_txt_attachment = open(
            os.path.join(here, 'mails', 'attachment.txt'), 'r').read()
        self.msg_utf8 = open(
            os.path.join(here, 'mails', 'utf8.txt'), 'r').read()

    def test_adding(self):
        mail = create(Builder('mail'))
        self.failUnless(IMail.providedBy(mail))

        self.assertEquals(u'no_subject', mail.title)

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='ftw.mail.mail')
        self.assertNotEquals(None, fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='ftw.mail.mail')
        schema = fti.lookupSchema()
        self.assertEquals(IMail, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='ftw.mail.mail')
        factory = fti.factory
        new_object = createObject(factory)
        self.failUnless(IMail.providedBy(new_object))
        self.assertEquals(u'no_subject', new_object.title)

    def test_attachment_info(self):
        mail = create(Builder('mail')
                      .with_message(self.msg_txt_attachment))

        self.assertEquals(
            ({'position': 1,
              'size': 7,
              'content-type': 'text/plain',
              'filename': 'B\xc3\xbccher.txt'}, ),
            mail.attachment_infos)

    def test_attachment_info_dict_mutations_are_not_stored(self):
        mail = create(Builder('mail')
                      .with_message(self.msg_txt_attachment))

        infos = mail.attachment_infos
        self.assertEquals(
            ({'position': 1,
              'size': 7,
              'content-type': 'text/plain',
              'filename': 'B\xc3\xbccher.txt'}, ),
            infos)
        infos[0]['FOO'] = 'BAR'
        infos[0]['filename'] = 'BAZ'

        self.assertEquals(
            ({'position': 1,
              'size': 7,
              'content-type': 'text/plain',
              'filename': 'B\xc3\xbccher.txt'}, ),
            mail.attachment_infos)

    def test_view(self):
        mail = create(Builder('mail'))

        mail.REQUEST.set('ACTUAL_URL', mail.absolute_url())
        view = mail.restrictedTraverse('@@view')
        view()

        self.assertEquals('', view.get_header('Subject'))
        self.assertEquals('', view.html_safe_body())

    def test_attachments(self):
        mail = create(Builder('mail')
                      .with_message(self.msg_txt_attachment))

        mail.REQUEST.set('ACTUAL_URL', mail.absolute_url())
        view = mail.restrictedTraverse('@@view')
        view()

        attachments = view.attachments()
        self.assertEquals(1, len(attachments))
        self.failUnless('icon' in attachments[0])

    def test_get_attachment(self):
        mail = create(Builder('mail'))

        view = mail.restrictedTraverse('@@get_attachment')
        self.assertRaises(NotFound, view)

    def test_setting_title(self):
        # Try setting the title property
        # This is not supposed to change the title,
        # since that is always read from the subject,
        # but it shouldn't fail with an AttributeError

        mail = create(Builder('mail'))
        self.assertEquals(u'no_subject', mail.title)

        mail.title = "New Title"
        self.assertEquals(u'no_subject', mail.title)

    def test_message_field_is_marked_as_primary_field(self):
        mail = create(Builder('mail'))
        self.assertEquals(IMail['message'], IPrimaryFieldInfo(mail).field)

    def test_header_cache_is_invalidated(self):
        mail = create(Builder('mail')
                      .with_message(self.msg_txt_attachment))
        self.assertEquals('Attachment Test',
                          mail.get_header('Subject'))

        mail.message = NamedBlobFile(
            data=self.msg_utf8,
            contentType='message/rfc822',
            filename=u'message.eml')
        self.assertEquals('Die B\xc3\xbcrgschaft',
                          mail.get_header('Subject'))

    def test_referenceing_mails_from_archetypes_objects(self):
        mail = create(Builder('mail'))
        page = create(Builder('page').having(relatedItems=[IUUID(mail)]))
        self.assertTrue(page)

    # def test_special(self):
    #     here = os.path.dirname(__file__)
    #     msg_txt = open(os.path.join(here, 'mails', 'cipra.txt'), 'r').read()
    #     fti = getUtility(IDexterityFTI, name='ftw.mail.mail')
    #     schema = fti.lookupSchema()
    #     field_type = getFields(schema)['message']._type
    #     obj = createContentInContainer(self.portal, 'ftw.mail.mail',
    #                message=field_type(data=msg_txt,
    #                contentType='message/rfc822', filename='message.eml'))
    #     m1 = self.portal[obj.getId()]
    #     view = m1.restrictedTraverse('@@view')
    #     view()



from zope.interface import Interface
from zope import schema


class IMailInbound(Interface):
    """ Inbound Mail processor
    """

    def msg():
        """ Returns an email.Message instance of the mail that was sent
            to the inbound mail processor.
        """

    def sender():
        """ Returns the senders email address who sent the mail to the
            inbound mail processor.
        """
    def recipient():
        """ get the email address of the recipient
        """


class IEmailAddress(Interface):
    """ Returns the email address for an object or the object an email
    email address
    """

    def get_object_for_email(email):
        """ Extract the important data of an email address and returns the
            object
        """

    def get_email_for_object(obj, domain):
        """ Returns the generated email address for an object
        """


class IMailSettings(Interface):

    unwrap_mail = schema.Bool(
        title=u'Unwrap Forwarded Mails',
        default=True,
        description=u'If enabled, mails containing an attached mail are '
                    u'unwrapped and only the attached mail is stored.'
    )

    mail_domain = schema.TextLine(
        title=u"Mail domain",
        description=u'Enter the mail domain which will be used \
for sending mails into this site.',
        default=u'example.org')



from ftw.mail.config import EXIT_CODES


class MailInboundException(Exception):
    """ An exception indicating an error occured while processing
        an inbound mail.
    """
    def __init__(self, exitcode, errormsg):
        self.exitcode = exitcode
        self.errormsg = errormsg

    def __str__(self):
        return '%s:%s' % (self.exitcode, self.errormsg)


class NoInput(MailInboundException):

    def __init__(self):
        MailInboundException.__init__(
            self, EXIT_CODES['NOINPUT'], 'No mail message supplied.')


class InvalidMessage(MailInboundException):

    def __init__(self):
        MailInboundException.__init__(
            self, EXIT_CODES['DATAERR'], 'Invalid mail message supplied.')


class NoSenderFound(MailInboundException):

    def __init__(self, mail):
        self.mail = mail
        MailInboundException.__init__(
            self, EXIT_CODES['NOPERM'], 'Could not extract sender email.')


class UnknownSender(MailInboundException):

    def __init__(self, mail):
        self.mail = mail
        MailInboundException.__init__(
            self, EXIT_CODES['NOPERM'], 'Unknown sender. Permission denied.')


class PermissionDenied(MailInboundException):

    def __init__(self, mail, user):
        self.mail = mail
        self.user = user
        MailInboundException.__init__(
            self, EXIT_CODES['NOPERM'],
            'Unable to create message. Permission denied.')


class DisallowedSubobjectType(MailInboundException):

    def __init__(self, mail, user):
        self.mail = mail
        self.user = user
        MailInboundException.__init__(
            self, EXIT_CODES['NOPERM'],
            'Disallowed subobject type. Permission denied.')


class DestinationDoesNotExist(MailInboundException):

    def __init__(self, emailaddress):
        self.emailaddress = emailaddress
        MailInboundException.__init__(
            self, EXIT_CODES['CANTCREAT'],
            'Destination does not exist.')



from ftw.upgrade import UpgradeStep


class UpdatePersistentCaches(UpgradeStep):

    def __call__(self):
        query = {'object_provides': ['ftw.mail.mail.IMail']}
        for mail in self.objects(query, 'Update mail caches'):
            # reset the message to trigger updating of all caches
            mail.Title()  # Fucking caches.
            mail.message = vars(mail).get('message')



from ftw.upgrade import UpgradeStep
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


class RemoveValidateSender(UpgradeStep):

    def __call__(self):
        registry = getUtility(IRegistry)
        key = 'ftw.mail.interfaces.IMailSettings.validate_sender'
        del registry.records[key]
